package DNS::Zone::Struct::Common;

our $DATE = '2019-09-09'; # DATE
our $VERSION = '0.002'; # VERSION
1;
# ABSTRACT: Common routines related to DNS zone structure

__END__

=pod

=encoding UTF-8

=head1 NAME

DNS::Zone::Struct::Common - Common routines related to DNS zone structure

=head1 VERSION

This document describes version 0.002 of DNS::Zone::Struct::Common (from Perl distribution DNS-Zone-Struct-Common), released on 2019-09-09.

=head1 HOMEPAGE

Please visit the project's homepage at L<https://metacpan.org/release/DNS-Zone-Struct-Common>.

=head1 SOURCE

Source repository is at L<https://github.com/perlancar/perl-DNS-Zone-Struct-Common>.

=head1 BUGS

Please report any bugs or feature requests on the bugtracker website L<https://rt.cpan.org/Public/Dist/Display.html?Name=DNS-Zone-Struct-Common>

When submitting a bug or request, please include a test-file or a
patch to an existing test-file that illustrates the bug or desired
feature.

=head1 AUTHOR

perlancar <perlancar@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2019 by perlancar@cpan.org.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
